@@ -3,13 +3,15 @@ use super::Optimizer;
 use std::ops::Deref;
 use std::ops::DerefMut;
 use swc_atoms::JsWord;
-use swc_common::comments::Comment;
-use swc_common::comments::CommentKind;
-use swc_common::Mark;
 use swc_common::Span;
 use swc_ecma_ast::*;
 use swc_ecma_utils::prop_name_eq;
 use swc_ecma_utils::ExprExt;
+use swc_ecma_utils::Id;
+use swc_ecma_visit::noop_visit_mut_type;
+use swc_ecma_visit::VisitMut;
+use swc_ecma_visit::VisitMutWith;
+use unicode_xid::UnicodeXID;
 
 impl<'b> Optimizer<'b> {
     pub(super) fn line_col(&self, span: Span) -> String {
@@ -73,79 +75,17 @@ impl<'b> Optimizer<'b> {
 
     /// Check for `/** @const */`.
     pub(super) fn has_const_ann(&self, span: Span) -> bool {
-        self.find_comment(span, |c| {
-            if c.kind == CommentKind::Block {
-                if !c.text.starts_with('*') {
-                    return false;
-                }
-                let t = c.text[1..].trim();
-                //
-                if t.starts_with("@const") {
-                    return true;
-                }
-            }
-
-            false
-        })
+        span.has_mark(self.marks.const_ann)
     }
 
     /// Check for `/*#__NOINLINE__*/`
     pub(super) fn has_noinline(&self, span: Span) -> bool {
-        self.has_flag(span, "NOINLINE")
-    }
-
-    fn find_comment<F>(&self, span: Span, mut op: F) -> bool
-    where
-        F: FnMut(&Comment) -> bool,
-    {
-        let mut found = false;
-        if let Some(comments) = self.comments {
-            let cs = comments.get_leading(span.lo);
-            if let Some(cs) = cs {
-                for c in &cs {
-                    found |= op(&c);
-                    if found {
-                        break;
-                    }
-                }
-            }
-        }
-
-        found
-    }
-
-    fn has_flag(&self, span: Span, text: &'static str) -> bool {
-        self.find_comment(span, |c| {
-            if c.kind == CommentKind::Block {
-                //
-                if c.text.len() == (text.len() + 5)
-                    && c.text.starts_with("#__")
-                    && c.text.ends_with("__")
-                    && text == &c.text[3..c.text.len() - 2]
-                {
-                    return true;
-                }
-            }
-
-            false
-        })
+        span.has_mark(self.marks.noinline)
     }
 
     #[allow(unused)]
     pub(super) fn is_done(&mut self, span: Span) -> bool {
-        let mut ctxt = span.ctxt;
-        if ctxt == self.done_ctxt {
-            return true;
-        }
-        loop {
-            let mark = ctxt.remove_mark();
-            if mark == Mark::root() {
-                return false;
-            }
-            if mark == self.done {
-                return true;
-            }
-        }
+        span.has_mark(self.done)
     }
 
     /// RAII guard to change context temporarically
@@ -271,3 +211,57 @@ pub(crate) fn is_valid_for_lhs(e: &Expr) -> bool {
         _ => true,
     }
 }
+
+pub(crate) fn is_directive(e: &Stmt) -> bool {
+    match e {
+        Stmt::Expr(s) => match &*s.expr {
+            Expr::Lit(Lit::Str(Str { value, .. })) => value.starts_with("use "),
+            _ => false,
+        },
+        _ => false,
+    }
+}
+
+pub(crate) fn is_valid_identifier(s: &str, ascii_only: bool) -> bool {
+    if ascii_only {
+        if s.chars().any(|c| !c.is_ascii()) {
+            return false;
+        }
+    }
+
+    s.starts_with(|c: char| c.is_xid_start())
+        && s.chars().all(|c: char| c.is_xid_continue())
+        && !s.contains("𝒶")
+        && !s.is_reserved()
+}
+
+pub(crate) fn replace_id_with_expr<N>(node: &mut N, from: Id, to: Box<Expr>)
+where
+    N: VisitMutWith<ExprReplacer>,
+{
+    node.visit_mut_with(&mut ExprReplacer { from, to: Some(to) })
+}
+
+pub(crate) struct ExprReplacer {
+    from: Id,
+    to: Option<Box<Expr>>,
+}
+
+impl VisitMut for ExprReplacer {
+    noop_visit_mut_type!();
+
+    fn visit_mut_expr(&mut self, e: &mut Expr) {
+        e.visit_mut_children_with(self);
+
+        match e {
+            Expr::Ident(i) => {
+                if self.from.0 == i.sym && self.from.1 == i.span.ctxt {
+                    if let Some(new) = self.to.take() {
+                        *e = *new;
+                    }
+                }
+            }
+            _ => {}
+        }
+    }
+}
